Skip to contents

These options will be used on objects of class 'col_spec'.

Usage

opts_col_spec(constructor = c("cols", "next", "list"), ...)

Arguments

constructor

String. Name of the function used to construct the object.

...

Additional options used by user defined constructors through the opts object

Value

An object of class <constructive_options/constructive_options_col_spec>

Details

Depending on constructor, we construct the object as follows:

  • "cols" (default): We build the object using readr::cols(), or readr::cols_only() when the default collector is readr::col_skip(). The .default argument is omitted when it's readr::col_guess().

  • "next" : Use the constructor for the next supported class. Call .class2() on the object to see in which order the methods will be tried. This will usually be equivalent to "list"

  • "list" : We define as a list and repair attributes

Use opts_collector() to tweak the construction of the column specifications.