Skip to contents

These options will be used on objects of class 'grouped_ts'.

Usage

opts_grouped_ts(constructor = c("group_by", "next"), ...)

Arguments

constructor

String. Name of the function used to construct the object, see Details section.

...

Additional options used by user defined constructors through the opts object

Value

An object of class <constructive_options/constructive_options_grouped_ts>

Details

Depending on constructor, we construct the object as follows:

  • "group_by" (default): Construct the ungrouped tsibble and pipe it into dplyr::group_by(), and into tsibble::index_by() if the object is grouped by an index. Use opts_tbl_ts() to tweak the construction of the tsibble.

  • "next" : Use the constructor for the next supported class. Call .class2() on the object to see in which order the methods will be tried.