Skip to contents

These options will be used on objects of class 'sfg', simple feature geometries from the 'sf' package.

Usage

opts_sfg(constructor = c("st_sfg", "next"), ...)

Arguments

constructor

String. Name of the function used to construct the object, see Details section.

...

Additional options used by user defined constructors through the opts object

Value

An object of class <constructive_options/constructive_options_sfg>

Details

Depending on constructor, we construct the object as follows:

Other geometry types, such as "CIRCULARSTRING", don't have a dedicated constructor and are built with the next constructor.