These options will be used on objects of class 'data.table'.
opts_vctrs_list_of(constructor = c("list_of", "next", "list"), ...)An object of class <constructive_options/constructive_options_vctrs_list_of>
String. Name of the function used to construct the object, see Details section.
Additional options used by user defined constructors through the opts object
Depending on constructor, we construct the object as follows:
"list_of" (default): Wrap the column definitions in a list_of() call.
"list" : Use list() and treat the class as a regular attribute.