Skip to contents

These options will be used on objects of class 'xts'.

Usage

opts_xts(
  constructor = c("xts", "as.xts.matrix", "as.xts.data.frame", ".xts", "next"),
  ...
)

Arguments

constructor

String. Name of the function used to construct the object.

...

Additional options used by user defined constructors through the opts object

Value

An object of class <constructive_options/constructive_options_xts>

Details

Depending on constructor, we construct the object as follows:

  • "xts" (default): We build the object using xts::xts(), the index is provided as a Date or POSIXct vector depending on its class.

  • "as.xts.matrix": We build the object using xts::as.xts.matrix(), the index is provided through row names, which doesn't preserve its class and time zone.

  • "as.xts.data.frame": We build the object using xts::as.xts.data.frame(), this is probably the most readable option but couldn't be made the default constructor because it requires the 'xts' package to be installed .

  • ".xts": We build the object using xts::.xts().

  • "next": Use the constructor for the next supported class.