Skip to contents

These options will be used on objects of class 'yearmonth'.

Usage

opts_yearmonth(constructor = c("yearmonth", "next"), ...)

Arguments

constructor

String. Name of the function used to construct the object, see Details section.

...

Additional options used by user defined constructors through the opts object

Value

An object of class <constructive_options/constructive_options_yearmonth>

Details

Depending on constructor, we construct the object as follows:

  • "yearmonth" (default): We build the object using tsibble::yearmonth() on a string in the format "2024 Jan". If the object contains NAs or years that can't be parsed from a string, we use a Date object as an input instead.

  • "next" : Use the constructor for the next supported class. Call .class2() on the object to see in which order the methods will be tried.