These options will be used on objects of class 'yearquarter'.
Usage
opts_yearquarter(constructor = c("yearquarter", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"yearquarter"(default): We build the object usingtsibble::yearquarter()on a string in the format"2024 Q1". If the object containsNAs or years that can't be parsed from a string, we use aDateobject as an input instead."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.