These options will be used on objects of class 'yearweek'.
Usage
opts_yearweek(constructor = c("yearweek", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"yearweek"(default): We build the object usingtsibble::yearweek()on a string in the format"2024 W01". If the object containsNAs or years that can't be parsed from a string, we use aDateobject as an input instead."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.