These options will be used on objects of class 'bbox'.
Usage
opts_bbox(constructor = c("st_bbox", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"st_bbox"(default): We build the object usingsf::st_bbox()on a named double vector, with acrsargument if relevant."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.