These options will be used on objects of class 'col_spec'.
Usage
opts_col_spec(constructor = c("cols", "next", "list"), ...)Details
Depending on constructor, we construct the object as follows:
"cols"(default): We build the object usingreadr::cols(), orreadr::cols_only()when the default collector isreadr::col_skip(). The.defaultargument is omitted when it'sreadr::col_guess()."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried. This will usually be equivalent to"list""list": We define as a list and repair attributes
Use opts_collector() to tweak the construction of the column specifications.