These options will be used on objects of class 'collector', i.e. column
specifications as created by readr::col_double(), readr::col_date(),
readr::col_factor() etc. They all share the class 'collector' and a subclass
'collector_readr::col_<type>().
Usage
opts_collector(constructor = c("col", "next", "list"), ...)Details
Depending on constructor, we construct the object as follows:
"col"(default): We build the object using the relevantreadr::col_*()function, e.g.readr::col_double()orreadr::col_date(format = "%Y"), providing only non default arguments."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried. This will usually be equivalent to"list""list": We define as a list and repair attributes