These options will be used on objects of class 'date_names'.
Usage
opts_date_names(
constructor = c("date_names_lang", "date_names", "next", "list"),
...
)Details
Depending on constructor, we construct the object as follows:
"date_names_lang"(default): We build the object usingreadr::date_names_lang()if the object matches one of the languages listed byreadr::date_names_langs(), and fall back to the"date_names"constructor otherwise."date_names": We build the object usingreadr::date_names(), providing only non default arguments."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried. This will usually be equivalent to"list""list": We define as a list and repair attributes