These options will be used on objects of class 'glue'.
Usage
opts_glue(constructor = c("as_glue", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"as_glue"(default): Useglue::as_glue()on a character vector."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.
Use opts_character() to tweak the construction of the character vector
constructed as part of the glue construction.