These options will be used on objects of class 'grouped_ts'.
Usage
opts_grouped_ts(constructor = c("group_by", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"group_by"(default): Construct the ungrouped tsibble and pipe it intodplyr::group_by(), and intotsibble::index_by()if the object is grouped by an index. Useopts_tbl_ts()to tweak the construction of the tsibble."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.