These options will be used on objects of class 'hms'.
Usage
opts_hms(constructor = c("as_hms", "hms", "new_hms", "next", "double"), ...)Details
Depending on constructor, we construct the object as follows:
"as_hms"(default): We build the object usinghms::as_hms()on a character vector in the format"12:34:56". If some values cannot be reproduced exactly this way (e.g. negative values, durations of more than 24 hours, or some fractional seconds) we fall back to the"hms"constructor."hms": We build the object usinghms::hms()onseconds,minutesandhoursarguments. If the decomposition is not exact we provide only thesecondsargument."new_hms": We build the object usinghms::new_hms()on a double vector."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried."double": We define as an atomic vector and repair attributes.