These options will be used on objects of class 'locale'.
Usage
opts_locale(constructor = c("locale", "next", "list"), ...)Details
Depending on constructor, we construct the object as follows:
"locale"(default): We build the object usingreadr::locale(), providing only non default arguments.date_namesis provided as a language code when possible."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried. This will usually be equivalent to"list""list": We define as a list and repair attributes
Use opts_date_names() to tweak the construction of the date_names element,
opts_date_names("date_names") will prevent the use of a language code.