These options will be used on objects of class 'mixed_units'.
Usage
opts_mixed_units(constructor = c("mixed_units", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"mixed_units"(default): We build the object usingunits::mixed_units()on a double vector and a character vector of unit strings as given byunits::deparse_unit()."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.