These options will be used on objects of class 'sf', simple feature data frames from the 'sf' package.
Usage
opts_sf(constructor = c("st_sf", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"st_sf"(default): We build the object usingsf::st_sf()on the columns, or on a tibble if the object is also a tibble, withsf_column_nameandagrarguments if relevant."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.
sf::st_sf() places the geometry columns last and doesn't support list
columns or names that clash with its arguments unless the input is a tibble,
in these cases we use the next constructor.