These options will be used on objects of class 'sfc', simple feature geometry list-columns from the 'sf' package.
Usage
opts_sfc(constructor = c("st_sfc", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"st_sfc"(default): We build the object usingsf::st_sfc()on the geometries, withcrsandprecisionarguments if relevant. The other attributes ("bbox", "n_empty", "classes"...) are computed bysf::st_sfc()and are repaired only if they don't match."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.