These options will be used on objects of class 'sfg', simple feature geometries from the 'sf' package.
Usage
opts_sfg(constructor = c("st_sfg", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"st_sfg"(default): We build the object using the sf constructor matching the geometry type, i.e.sf::st_point(),sf::st_multipoint(),sf::st_linestring(),sf::st_multilinestring(),sf::st_polygon(),sf::st_multipolygon()orsf::st_geometrycollection(), with adimargument when the dimension can't be inferred from the coordinates."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.
Other geometry types, such as "CIRCULARSTRING", don't have a dedicated constructor and are built with the next constructor.