These options will be used on objects of class 'tbl_ts', also known as tsibbles.
Usage
opts_tbl_ts(constructor = c("tsibble", "as_tsibble", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"tsibble"(default): Wrap the column definitions in atsibble::tsibble()call. If some column names conflict with the arguments oftsibble::tsibble()we fall back to"as_tsibble"."as_tsibble": Construct the underlying tibble and pipe it intotsibble::as_tsibble(). Useopts_tbl_df()to tweak the construction of the tibble."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.
The "interval" attribute is always recomputed by 'tsibble', we only set
regular = FALSE for irregular tsibbles.