These options will be used on objects of class 'units'.
Usage
opts_units(constructor = c("set_units", "as_units", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"set_units"(default): We build the object usingunits::set_units(x, value, mode = "standard")wherevalueis the unit string as given byunits::deparse_unit()."as_units": We build the object usingunits::as_units(x, value)."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.