These options will be used on objects of class 'yearmonth'.
Usage
opts_yearmonth(constructor = c("yearmonth", "next"), ...)Details
Depending on constructor, we construct the object as follows:
"yearmonth"(default): We build the object usingtsibble::yearmonth()on a string in the format"2024 Jan". If the object containsNAs or years that can't be parsed from a string, we use aDateobject as an input instead."next": Use the constructor for the next supported class. Call.class2()on the object to see in which order the methods will be tried.